SP     DATA  081H  ;STACK POINTER
DPL    DATA  082H  ;DATA POINTER - LOW BYTE
DPH    DATA  083H  ;DATA POINTER - HIGH BYTE
PCON   DATA  087H  ;POWER CONTROL
TCON   DATA  088H  ;TIMER CONTROL
TMOD   DATA  089H  ;TIMER MODE
TL0    DATA  08AH  ;TIMER 0 - LOW BYTE
TL1    DATA  08BH  ;TIMER 1 - LOW BYTE
TH0    DATA  08CH  ;TIMER 0 - HIGH BYTE
TH1    DATA  08DH  ;TIMER 1 - HIGH BYTE
P1     DATA  090H  ;PORT 1
SCON   DATA  098H  ;SERIAL PORT CONTROL
SBUF   DATA  099H  ;SERIAL PORT BUFFER
IE     DATA  0A8H  ;INTERRUPT ENABLE
P3     DATA  0B0H  ;PORT 3
IP     DATA  0B8H  ;INTERRUPT PRIORITY
PSW    DATA  0D0H  ;PROGRAM STATUS WORD
ACC    DATA  0E0H  ;ACCUMULATOR
B      DATA  0F0H  ;MULTIPLICATION REGISTER
IT0    BIT   088H  ;TCON.0 - EXT. INTERRUPT 0 TYPE
IE0    BIT   089H  ;TCON.1 - EXT. INTERRUPT 0 EDGE FLAG
IT1    BIT   08AH  ;TCON.2 - EXT. INTERRUPT 1 TYPE
IE1    BIT   08BH  ;TCON.3 - EXT. INTERRUPT 1 EDGE FLAG
TR0    BIT   08CH  ;TCON.4 - TIMER 0 ON/OFF CONTROL
TF0    BIT   08DH  ;TCON.5 - TIMER 0 OVERFLOW FLAG
TR1    BIT   08EH  ;TCON.6 - TIMER 1 ON/OFF CONTROL
TF1    BIT   08FH  ;TCON.7 - TIMER 1 OVERFLOW FLAG
RI     BIT   098H  ;SCON.0 - RECEIVE INTERRUPT FLAG
TI     BIT   099H  ;SCON.1 - TRANSMIT INTERRUPT FLAG
RB8    BIT   09AH  ;SCON.2 - RECEIVE BIT 8
TB8    BIT   09BH  ;SCON.3 - TRANSMIT BIT 8
REN    BIT   09CH  ;SCON.4 - RECEIVE ENABLE
SM2    BIT   09DH  ;SCON.5 - SERIAL MODE CONTROL BIT 2
SM1    BIT   09EH  ;SCON.6 - SERIAL MODE CONTROL BIT 1
SM0    BIT   09FH  ;SCON.7 - SERIAL MODE CONTROL BIT 0
EX0    BIT   0A8H  ;IE.0 - EXTERNAL INTERRUPT 0 ENABLE
ET0    BIT   0A9H  ;IE.1 - TIMER 0 INTERRUPT ENABLE
EX1    BIT   0AAH  ;IE.2 - EXTERNAL INTERRUPT 1 ENABLE
ET1    BIT   0ABH  ;IE.3 - TIMER 1 INTERRUPT ENABLE
ES     BIT   0ACH  ;IE.4 - SERIAL PORT INTERRUPT ENABLE
EA     BIT   0AFH  ;IE.7 - GLOBAL INTERRUPT ENABLE
RXD    BIT   0B0H  ;P3.0 - SERIAL PORT RECEIVE INPUT
TXD    BIT   0B1H  ;P3.1 - SERIAL PORT TRANSMIT OUTPUT
INT0   BIT   0B2H  ;P3.2 - EXTERNAL INTERRUPT 0 INPUT
INT1   BIT   0B3H  ;P3.3 - EXTERNAL INTERRUPT 1 INPUT
T0     BIT   0B4H  ;P3.4 - TIMER 0 COUNT INPUT
T1     BIT   0B5H  ;P3.5 - TIMER 1 COUNT INPUT
WR     BIT   0B6H  ;P3.6 - WRITE CONTROL FOR EXT. MEMORY
RD     BIT   0B7H  ;P3.7 - READ CONTROL FOR EXT. MEMORY
PX0    BIT   0B8H  ;IP.0 - EXTERNAL INTERRUPT 0 PRIORITY
PT0    BIT   0B9H  ;IP.1 - TIMER 0 PRIORITY
PX1    BIT   0BAH  ;IP.2 - EXTERNAL INTERRUPT 1 PRIORITY
PT1    BIT   0BBH  ;IP.3 - TIMER 1 PRIORITY
PS     BIT   0BCH  ;IP.4 - SERIAL PORT PRIORITY
P      BIT   0D0H  ;PSW.0 - ACCUMULATOR PARITY FLAG
OV     BIT   0D2H  ;PSW.2 - OVERFLOW FLAG
RS0    BIT   0D3H  ;PSW.3 - REGISTER BANK SELECT 0
RS1    BIT   0D4H  ;PSW.4 - REGISTER BANK SELECT 1
F0     BIT   0D5H  ;PSW.5 - FLAG 0
AC     BIT   0D6H  ;PSW.6 - AUXILIARY CARRY FLAG
CY     BIT   0D7H  ;PSW.7 - CARRY FLAG

SDA             equ     p3.5
SCL             equ     p3.7

BAUD_9600 EQU 0fdh ; 9600 baud (for 11059 KGz occilator )
baud_1200 equ 0d0h ; 1200      (for 11059 KGz occilator )
DSEG AT 20h
ORG 60h ; stack origin
stack: DS 20h ; stack depth
;Macros...    

CSEG
ORG 0000h ; power on/reset vector
jmp init
ORG 0003h ; external interrupt 0 vector
reti ; undefined
ORG 000Bh ; timer 0 overflow vector
reti ; undefined
ORG 0013h ; external interrupt 1 vector
reti ; undefined
ORG 001Bh ; timer 1 overflow vector
reti ; undefined
ORG 0023h ; serial I/O interrupt vector
reti ; undefined
ORG 0040h ; begin code space
USING 0 ; register bank 0
init:
mov sp, #(stack-1) ; initialize stack pointer
call initialize ; initialize controller registers
   setb TI;set transmit interrupt flag (kludge for first use)
   setb ES; enable serial ints
mov r6,#00000000b; set counter of EEPROM address to zero
mov a,p1;

anl a,#01000000b;testing button command;
jnz m2;jump if button isn't pushed /to read eeprom in circle
m1:mov a, #1;
   call delay_sec; delay 1 sec between AD conversions
mov a,r6;
mov r5,a;
   mov a, #10101010b;
call send_byte;
;setb p1.3; select adc and status nibble
call adc_ints;do ad conversion with subsequent data transmission via UART 
;clr p1.3; select contact with eeprom

cjne r6,#11111111b,m3;	define data address, if less 255 record current data
jmp m1;
m3:
mov r7,#10100000b;set up slave address
mov r3,th0; LSB to r3 
		mov	b, #120		; retry counter
	x52:
		mov	a, r5	; programmable address
		call	write_byte	; try to write
		jnc	x53		; jump if write OK

		djnz	b, x52		; try again
		setb	c		; set timeout error flag

	x53:

	  mov a, #15; retention time to record data in eeprom
	  call delay_ms;delay 2 msec before to send MSB
inc r6;
mov a,r6;
mov r5,a;
cjne r6,#11111111b,m4;	define data address
jmp m1;
m4: mov r3,tl0; MSB to r3
		mov	b, #120		; retry counter
	x54:
		mov	a, r5	; programmable address
		call	write_byte	; try to write
		jnc	x55		; jump if write OK

		djnz	b, x54		; try again
		setb	c		; set timeout error flag

	x55:

inc r6;
jmp m1;

m2:;to read data from eeprom only and transmit them via UART until r5 overflow
;clr p1.3; select contact with eeprom
mov a,r6;
mov r5,a;
mov r7,#10100000b;set up slave address
		mov	b, #120		; retry counter
	x81:
		mov	a, r5	; programmable address
		call	read_random	; try to read
		jnc	x82		; jump if read OK

		djnz	b, x81		; try again
	x82:


call send_byte;
	  mov a, #4;
	  call delay_ms;delay 4 msec before to send MSB
inc r6;define data address
cjne r6,#11111111b,m2;
jmp m1;

initialize:
; Initialize controller registers and I/O lines.
mov IE, #0 ; deactivate all interrupts
mov p1,#0ffh;deactivate output port p1
mov SCON, #01000000b; serial port mode 1
mov TMOD, #00100000b; timer 1 8-bit auto-reload,
; timer 0 8+5-bit
;mov TH1, #baud_1200;
mov TH1, #BAUD_9600 ; timer 1 reload value
mov TCON, #01000000b; start timer 1
setb EA     ;global int enable
ret

send_byte:
	; Wait for transmitter to clear and transmit byte in accumulator .
; Does not wait for transmitter to clear before returning.
	     jnb   TI, $       ; wait here for transmitter to clear
		clr   TI          ; clear transmit flag
		mov   SBUF, a  
ret


adc_ints:;one slope AD conversion with data transmition  via UART
	  mov a, #4;
          call delay_ms;delay 2 msec
mov th0,#00000000b;
mov tl0,#00000000b;
clr p1.4;close disgarge transistor
  setb tcon.4;start timer 0
  jb p3.6, $;waiting comparator
	clr tcon.4;stop timer 0
	  setb p1.4;discharge capacitor
	  mov a,tl0; LSB to a
 call send_byte;five LSB first
	  mov a, #4;
	  call delay_ms;delay 2 msec before to send MSB
	  mov a, th0;
          call send_byte;
 ret;


delay_ms:
; Delay for 1 ms times the value in the accumulator.
push acc
push b
mov b, #0
dd:
djnz b, $ ; 500 us @ 12 MHz
djnz b, $ ; 500 us @ 12 MHz
djnz acc, dd
pop b
pop acc
ret

delay_sec:
; Delay for 1 s times the value in the accumulator.
push acc
push b
mov b, a
ddd:
mov a, #250
call delay_ms; 250 ms
call delay_ms; 500 ms
;call delay_ms; 750 ms
;call delay_ms; 1000 ms
djnz b, ddd
pop b
pop acc
ret

write_byte:

	; AT24Cxx Byte Write function.
	; Called with programmable address in A, byte address in
	; register pair ADDR_HI:ADDR_LO, data in register XDATA.
	; Does not wait for write cycle to complete.
	; Returns CY set to indicate that the bus is not available
	; or that the addressed device failed to acknowledge.
	; Destroys A.

		call	start
		jc	x49		; abort if bus not available

;		rl	a		; programmable address to bits 3:1
	mov	a,r7	; add fixed address

		clr	acc.0		; specify write operation
		call	shout		; send device address
		jc	x48		; abort if no acknowledge

		mov	a, r5	; send high byte of address
		call	shout		;
		jc	x48		; abort if no acknowledge


		mov	a, r3	; get data
		call	shout		; send data
		jc	x48		; abort if no acknowledge

		clr	c		; clear error flag
	x48:
		call	stop
	x49:
		ret


read_current:

	; AT24Cxx Current Address Read function.
	; Called with programmable address in A. Returns data in A.
	; Returns CY set to indicate that the bus is not available
	; or that the addressed device failed to acknowledge.

		call	start
		jc	x45		; abort if bus not available

;		rl	a		; programmable address to bits 3:1
	mov	a,r7	; add fixed address

		setb	acc.0		; specify read operation
		call	shout		; send device address
		jc	x44		; abort if no acknowledge

		call	shin		; receive data byte
		call	NAK		; do not acknowledge byte
		clr	c		; clear error flag
	x44:
		call	stop
	x45:
		ret


read_random:

	; AT24Cxx Random Read function.
	; Called with programmable address in r5, byte address in
	; register r5. Returns data in A.
	; Returns CY set to indicate that the bus is not available
	; or that the addressed device failed to acknowledge.

		push	b
		mov	b, r5		; save copy of programmable address

		; Send dummy write command to set internal address.

		call	start
		jc	x47		; abort if bus not available

;		rl	a		; programmable address to bits 3:1
	mov	a,r7	; add fixed address
		clr	acc.0		; specify write operation
		call	shout		; send device address
		jc	x46		; abort if no acknowledge

	mov	a,r5	; send byte of address
		call	shout		;
		jc	x46		; abort if no acknowledge

;		mov	a, addr_lo	; send low byte of address
;		call	shout		;
;		jc	x46		; abort if no acknowledge

		; Call Current Address Read function.

		mov	a, b		; get programmable address
		call	read_current
		jmp	x47		; exit
	x46:
		call	stop
	x47:
		pop	b
		ret


start:

	; Send START, defined as high-to-low SDA with SCL high.
	; Return with SCL, SDA low.
	; Returns CY set if bus is not available.

		setb	SDA
		setb	SCL

		; Verify bus available.

		jnb	SDA, x40	; jump if not high
		jnb	SCL, x40	; jump if not high

		nop			; enforce setup delay and cycle delay
		clr	SDA
		nop			; enforce hold delay
		nop			;
		nop			;
		nop			;
		nop			;
		clr	SCL

		clr	c		; clear error flag
		jmp	x41
	x40:
		setb	c		; set error flag
	x41:
		ret


stop:

	; Send STOP, defined as low-to-high SDA with SCL high.
	; SCL expected low on entry. Return with SCL, SDA high.

		clr	SDA
		nop			; enforce SCL low and data setup
		nop
		setb	SCL
		nop			; enforce setup delay
		nop			;
		nop			;
		nop			;
		nop			;
		setb	SDA
		ret


shout:

	; Shift out a byte to the AT24Cxx, most significant bit first.
	; SCL, SDA expected low on entry. Return with SCL low.
	; Called with data to send in A.
	; Returns CY set to indicate failure by slave to acknowledge.
	; Destroys A.

		push	b
		mov	b, #8		; bit counter
	x42:
		rlc	a		; move bit into CY
		mov	SDA, c		; output bit
		nop			; enforce SCL low and data setup
		setb	SCL		; raise clock
		nop			; enforce SCL high
		nop			;
		nop			;
		nop			;
		clr	SCL		; drop clock
		djnz	b, x42		; next bit

		setb	SDA		; release SDA for ACK
		nop			; enforce SCL low and tAA
		nop			;
		setb	SCL		; raise ACK clock
		nop			; enforce SCL high
		nop			;
		nop			;
		nop			;
		mov	c, SDA		; get ACK bit
		clr	SCL		; drop ACK clock

		pop	b
		ret


shin:

	; Shift in a byte from the AT24Cxx, most significant bit first.
	; SCL expected low on entry. Return with SCL low.
	; Returns received data byte in A.

		setb	SDA		; make SDA an input

		push	b
		mov	b, #8		; bit count
	x43:
		nop			; enforce SCL low and data setup
		nop			;
		nop			;
		setb	SCL		; raise clock
		nop			; enforce SCL high
		nop			;
		mov	c, SDA		; input bit
		rlc	a		; move bit into byte
		clr	SCL		; drop clock
		djnz	b, x43		; next bit

		pop	b
		ret


ACK:

	; Clock out an acknowledge bit (low).
	; SCL expected low on entry. Return with SCL, SDA low.

		clr	SDA		; ACK bit
		nop			; enforce SCL low and data setup
		nop			;
		setb	SCL		; raise clock
		nop			; enforce SCL high
		nop			;
		nop			;
		nop			;
		clr	SCL		; drop clock
		ret


NAK:

	; Clock out a negative acknowledge bit (high).
	; SCL expected low on entry. Return with SCL low, SDA high.

		setb	SDA		; NAK bit
		nop			; enforce SCL low and data setup
		nop			;
		setb	SCL		; raise clock
		nop			; enforce SCL high
		nop			;
		nop			;
		nop			;
		clr	SCL		; drop clock
		ret


END
